@@ -3,7 +3,7 @@
 /*
  *  Nautilus
  *
- *  Copyright (C) 2004 Red Hat, Inc.
+ *  Copyright (C) 2004, 2011 Red Hat, Inc.
  *  Copyright (C) 2003 Marco Pesenti Gritti
  *
  *  Nautilus is free software; you can redistribute it and/or
@@ -24,30 +24,29 @@
  *
  *  Authors: Alexander Larsson <alexl@redhat.com>
  *           Marco Pesenti Gritti
+ *           Cosimo Cecchi <cosimoc@redhat.com>
  *
  */
 
 #include <config.h>
 
 #include "nautilus-navigation-action.h"
-#include "nautilus-navigation-window.h"
-#include "nautilus-window-private.h"
-#include "nautilus-navigation-window-slot.h"
+
+#include "nautilus-window.h"
+
 #include <gtk/gtk.h>
 #include <eel/eel-gtk-extensions.h>
 
-static void nautilus_navigation_action_init       (NautilusNavigationAction *action);
-static void nautilus_navigation_action_class_init (NautilusNavigationActionClass *class);
-
-static GObjectClass *parent_class = NULL;
-
+G_DEFINE_TYPE (NautilusNavigationAction, nautilus_navigation_action, GTK_TYPE_ACTION);
 #define NAUTILUS_NAVIGATION_ACTION_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), NAUTILUS_TYPE_NAVIGATION_ACTION, NautilusNavigationActionPrivate))
 
 struct NautilusNavigationActionPrivate
 {
-	NautilusNavigationWindow *window;
+	NautilusWindow *window;
 	NautilusNavigationDirection direction;
 	char *arrow_tooltip;
+
+        guint popup_timeout_id;
 };
 
 enum
@@ -58,32 +57,6 @@ enum
 	PROP_WINDOW
 };
 
-GType
-nautilus_navigation_action_get_type (void)
-{
-	static GType type = 0;
-
-	if (type == 0) {
-		const GTypeInfo type_info = {
-			sizeof (NautilusNavigationActionClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) nautilus_navigation_action_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL,
-			sizeof (NautilusNavigationAction),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) nautilus_navigation_action_init,
-		};
-		
-		type = g_type_register_static (GTK_TYPE_ACTION,
-					       "NautilusNavigationAction",
-					       &type_info, 0);
-	}
-
-	return type;
-}
-
 static gboolean
 should_open_in_new_tab (void)
 {
@@ -102,44 +75,42 @@ should_open_in_new_tab (void)
 
 static void
 activate_back_or_forward_menu_item (GtkMenuItem *menu_item, 
-				    NautilusNavigationWindow *window,
+				    NautilusWindow *window,
 				    gboolean back)
 {
 	int index;
 	
 	g_assert (GTK_IS_MENU_ITEM (menu_item));
-	g_assert (NAUTILUS_IS_NAVIGATION_WINDOW (window));
 
 	index = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menu_item), "user_data"));
 
-	nautilus_navigation_window_back_or_forward (window, back, index, should_open_in_new_tab ());
+	nautilus_window_back_or_forward (window, back, index, should_open_in_new_tab ());
 }
 
 static void
-activate_back_menu_item_callback (GtkMenuItem *menu_item, NautilusNavigationWindow *window)
+activate_back_menu_item_callback (GtkMenuItem *menu_item,
+                                  NautilusWindow *window)
 {
 	activate_back_or_forward_menu_item (menu_item, window, TRUE);
 }
 
 static void
-activate_forward_menu_item_callback (GtkMenuItem *menu_item, NautilusNavigationWindow *window)
+activate_forward_menu_item_callback (GtkMenuItem *menu_item, NautilusWindow *window)
 {
 	activate_back_or_forward_menu_item (menu_item, window, FALSE);
 }
 
 static void
-fill_menu (NautilusNavigationWindow *window,
+fill_menu (NautilusWindow *window,
 	   GtkWidget *menu,
 	   gboolean back)
 {
-	NautilusNavigationWindowSlot *slot;
+	NautilusWindowSlot *slot;
 	GtkWidget *menu_item;
 	int index;
 	GList *list;
 
-	g_assert (NAUTILUS_IS_NAVIGATION_WINDOW (window));
-
-	slot = NAUTILUS_NAVIGATION_WINDOW_SLOT (NAUTILUS_WINDOW (window)->details->active_pane->active_slot);
+	slot = nautilus_window_get_active_slot (window);
 	
 	list = back ? slot->back_list : slot->forward_list;
 	index = 0;
@@ -160,27 +131,18 @@ fill_menu (NautilusNavigationWindow *window,
 }
 
 static void
-show_menu_callback (GtkMenuToolButton *button,
-		    NautilusNavigationAction *action)
+show_menu (NautilusNavigationAction *self,
+           guint button,
+           guint32 event_time)
 {
-	NautilusNavigationActionPrivate *p;
-	NautilusNavigationWindow *window;
+	NautilusWindow *window;
 	GtkWidget *menu;
-	GList *children;
-	GList *li;
 
-	p = action->priv;
-	window = action->priv->window;
+	window = self->priv->window;
 	
-	menu = gtk_menu_tool_button_get_menu (button);
-	
-	children = gtk_container_get_children (GTK_CONTAINER (menu));
-	for (li = children; li; li = li->next) {
-		gtk_container_remove (GTK_CONTAINER (menu), li->data);
-	}
-	g_list_free (children);
+	menu = gtk_menu_new ();
 
-	switch (p->direction) {
+	switch (self->priv->direction) {
 	case NAUTILUS_NAVIGATION_DIRECTION_FORWARD:
 		fill_menu (window, menu, FALSE);
 		break;
@@ -191,75 +153,133 @@ show_menu_callback (GtkMenuToolButton *button,
 		g_assert_not_reached ();
 		break;
 	}
+
+        gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
+                        button, event_time);
+}
+
+#define MENU_POPUP_TIMEOUT 1200
+
+static gboolean
+popup_menu_timeout_cb (gpointer data)
+{
+        NautilusNavigationAction *self = data;
+
+        show_menu (self, 1, gtk_get_current_event_time ());
+
+        return FALSE;
+}
+
+static void
+unschedule_menu_popup_timeout (NautilusNavigationAction *self)
+{
+        if (self->priv->popup_timeout_id != 0) {
+                g_source_remove (self->priv->popup_timeout_id);
+                self->priv->popup_timeout_id = 0;
+        }
+}
+
+static void
+schedule_menu_popup_timeout (NautilusNavigationAction *self)
+{
+        /* unschedule any previous timeouts */
+        unschedule_menu_popup_timeout (self);
+
+        self->priv->popup_timeout_id =
+                g_timeout_add (MENU_POPUP_TIMEOUT,
+                               popup_menu_timeout_cb,
+                               self);
 }
 
 static gboolean
-proxy_button_press_event_cb (GtkButton *button,
-			     GdkEventButton *event,
-			     gpointer user_data)
+tool_button_press_cb (GtkButton *button,
+                      GdkEventButton *event,
+                      gpointer user_data)
 {
-	if (event->button == 2) {
-                g_signal_emit_by_name (button, "pressed", 0);
-	}
+        NautilusNavigationAction *self = user_data;
+
+        if (event->button == 3) {
+                /* right click */
+                show_menu (self, event->button, event->time);
+                return TRUE;
+        }
+
+        if (event->button == 1) {
+                schedule_menu_popup_timeout (self);
+        }
 
 	return FALSE;
 }
 
 static gboolean
-proxy_button_release_event_cb (GtkButton *button,
-			       GdkEventButton *event,
-			       gpointer user_data)
+tool_button_release_cb (GtkButton *button,
+                        GdkEventButton *event,
+                        gpointer user_data)
 {
-	if (event->button == 2) {
-                g_signal_emit_by_name (button, "released", 0);
-	}
+        NautilusNavigationAction *self = user_data;
 
-	return FALSE;
+        unschedule_menu_popup_timeout (self);
+        
+        return FALSE;
+}
+
+static GtkWidget *
+get_actual_button (GtkToolButton *tool_button)
+{
+	GList *children;
+	GtkWidget *button;
+
+	g_return_val_if_fail (GTK_IS_TOOL_BUTTON (tool_button), NULL);
+
+	children = gtk_container_get_children (GTK_CONTAINER (tool_button));
+	button = GTK_WIDGET (children->data);
+
+	g_list_free (children);
+
+	return button;
 }
 
 static void
-connect_proxy (GtkAction *action, GtkWidget *proxy)
+connect_proxy (GtkAction *action,
+               GtkWidget *proxy)
 {
-	if (GTK_IS_MENU_TOOL_BUTTON (proxy)) {
-		NautilusNavigationAction *naction = NAUTILUS_NAVIGATION_ACTION (action);
-		GtkMenuToolButton *button = GTK_MENU_TOOL_BUTTON (proxy);
-		GtkWidget *menu;
-		GtkWidget *child;
+        GtkToolButton *tool;
+        GtkWidget *button;
 
-		/* set an empty menu, so the arrow button becomes sensitive */
-		menu = gtk_menu_new ();
-		gtk_menu_tool_button_set_menu (button, menu);
+	if (GTK_IS_TOOL_BUTTON (proxy)) {
+                tool = GTK_TOOL_BUTTON (proxy);
+                button = get_actual_button (tool);
 
-		gtk_menu_tool_button_set_arrow_tooltip_text (button,
-							     naction->priv->arrow_tooltip);
-		
-		g_signal_connect (proxy, "show-menu",
-				  G_CALLBACK (show_menu_callback), action);
+                g_signal_connect (button, "button-press-event",
+                                  G_CALLBACK (tool_button_press_cb), action);
+                g_signal_connect (button, "button-release-event",
+                                  G_CALLBACK (tool_button_release_cb), action);
+        }
 
-		/* Make sure that middle click works. Note that there is some code duplication
-		 * between here and nautilus-window-menus.c */
-		child = eel_gtk_menu_tool_button_get_button (button);
-		g_signal_connect (child, "button-press-event", G_CALLBACK (proxy_button_press_event_cb), NULL);
-		g_signal_connect (child, "button-release-event", G_CALLBACK (proxy_button_release_event_cb), NULL);
-	}
-
-	(* GTK_ACTION_CLASS (parent_class)->connect_proxy) (action, proxy);
+	(* GTK_ACTION_CLASS (nautilus_navigation_action_parent_class)->connect_proxy) (action, proxy);
 }
 
 static void
-disconnect_proxy (GtkAction *action, GtkWidget *proxy)
+disconnect_proxy (GtkAction *action,
+                  GtkWidget *proxy)
 {
-	if (GTK_IS_MENU_TOOL_BUTTON (proxy)) {
-		GtkWidget *child;
-		
-		g_signal_handlers_disconnect_by_func (proxy, G_CALLBACK (show_menu_callback), action);
+        GtkToolButton *tool;
+        GtkWidget *button;
 
-		child = eel_gtk_menu_tool_button_get_button (GTK_MENU_TOOL_BUTTON (proxy));
-		g_signal_handlers_disconnect_by_func (child, G_CALLBACK (proxy_button_press_event_cb), NULL);
-		g_signal_handlers_disconnect_by_func (child, G_CALLBACK (proxy_button_release_event_cb), NULL);
+	if (GTK_IS_TOOL_BUTTON (proxy)) {
+                tool = GTK_TOOL_BUTTON (proxy);
+                button = get_actual_button (tool);
+
+                /* remove any possible timeout going on */
+                unschedule_menu_popup_timeout (NAUTILUS_NAVIGATION_ACTION (action));
+
+		g_signal_handlers_disconnect_by_func (button,
+                                                      G_CALLBACK (tool_button_press_cb), action);
+		g_signal_handlers_disconnect_by_func (button,
+                                                      G_CALLBACK (tool_button_release_cb), action);
 	}
 
-	(* GTK_ACTION_CLASS (parent_class)->disconnect_proxy) (action, proxy);
+	(* GTK_ACTION_CLASS (nautilus_navigation_action_parent_class)->disconnect_proxy) (action, proxy);
 }
 
 static void
@@ -267,9 +287,12 @@ nautilus_navigation_action_finalize (GObject *object)
 {
 	NautilusNavigationAction *action = NAUTILUS_NAVIGATION_ACTION (object);
 
+        /* remove any possible timeout going on */
+        unschedule_menu_popup_timeout (action);
+
 	g_free (action->priv->arrow_tooltip);
 
-	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
+	(* G_OBJECT_CLASS (nautilus_navigation_action_parent_class)->finalize) (object);
 }
 
 static void
@@ -292,7 +315,7 @@ nautilus_navigation_action_set_property (GObject *object,
 			nav->priv->direction = g_value_get_int (value);
 			break;
 		case PROP_WINDOW:
-			nav->priv->window = NAUTILUS_NAVIGATION_WINDOW (g_value_get_object (value));
+			nav->priv->window = g_value_get_object (value);
 			break;
 	}
 }
@@ -331,9 +354,7 @@ nautilus_navigation_action_class_init (NautilusNavigationActionClass *class)
 	object_class->set_property = nautilus_navigation_action_set_property;
 	object_class->get_property = nautilus_navigation_action_get_property;
 
-	parent_class = g_type_class_peek_parent (class);
-
-	action_class->toolbar_item_type = GTK_TYPE_MENU_TOOL_BUTTON;
+	action_class->toolbar_item_type = GTK_TYPE_TOOL_BUTTON;
 	action_class->connect_proxy = connect_proxy;
 	action_class->disconnect_proxy = disconnect_proxy;
 
@@ -358,7 +379,7 @@ nautilus_navigation_action_class_init (NautilusNavigationActionClass *class)
                                          g_param_spec_object ("window",
                                                               "Window",
                                                               "The navigation window",
-                                                              G_TYPE_OBJECT,
+                                                              NAUTILUS_TYPE_WINDOW,
                                                               G_PARAM_READWRITE));
 
 	g_type_class_add_private (object_class, sizeof(NautilusNavigationActionPrivate));
