@@ -147,7 +147,7 @@ typedef enum {
 
 static void  open_selected_bookmark                    (NautilusPlacesSidebar        *sidebar,
 							GtkTreeModel                 *model,
-							GtkTreePath                  *path,
+							GtkTreeIter                  *iter,
 							NautilusWindowOpenFlags flags);
 static void  nautilus_places_sidebar_style_set         (GtkWidget                    *widget,
 							GtkStyle                     *previous_style);
@@ -220,7 +220,7 @@ get_eject_icon (NautilusPlacesSidebar *sidebar,
 	GtkStyleContext *style;
 
 	icon_theme = gtk_icon_theme_get_default ();
-	icon_size = nautilus_get_icon_size_for_stock_size (GTK_ICON_SIZE_SMALL_TOOLBAR);
+	icon_size = nautilus_get_icon_size_for_stock_size (GTK_ICON_SIZE_MENU);
 	icon = g_themed_icon_new_with_default_fallbacks ("media-eject-symbolic");
 	icon_info = gtk_icon_theme_lookup_by_gicon (icon_theme, icon, icon_size, 0);
 
@@ -639,28 +639,25 @@ update_places (NautilusPlacesSidebar *sidebar)
 			       _("Computer"));
 
 	/* add built in bookmarks */
-	desktop_path = nautilus_get_desktop_directory ();
 
 	/* home folder */
-	if (strcmp (g_get_home_dir(), desktop_path) != 0) {
-		mount_uri = nautilus_get_home_directory_uri ();
-		icon = g_themed_icon_new (NAUTILUS_ICON_HOME);
-		last_iter = add_place (sidebar, PLACES_BUILT_IN,
-				       SECTION_COMPUTER,
-				       _("Home"), icon,
-				       mount_uri, NULL, NULL, NULL, 0,
-				       _("Open your personal folder"));
-		g_object_unref (icon);
-		compare_for_selection (sidebar,
-				       location, mount_uri, last_uri,
-				       &last_iter, &select_path);
-		g_free (mount_uri);
-	}
-
+	mount_uri = nautilus_get_home_directory_uri ();
+	icon = g_themed_icon_new (NAUTILUS_ICON_HOME);
+	last_iter = add_place (sidebar, PLACES_BUILT_IN,
+			       SECTION_COMPUTER,
+			       _("Home"), icon,
+			       mount_uri, NULL, NULL, NULL, 0,
+			       _("Open your personal folder"));
+	g_object_unref (icon);
+	compare_for_selection (sidebar,
+			       location, mount_uri, last_uri,
+			       &last_iter, &select_path);
+	g_free (mount_uri);
 
 	if (g_settings_get_boolean (gnome_background_preferences, NAUTILUS_PREFERENCES_SHOW_DESKTOP) &&
 	    !g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_DESKTOP_IS_HOME_DIR)) {
 		/* desktop */
+		desktop_path = nautilus_get_desktop_directory ();
 		mount_uri = g_filename_to_uri (desktop_path, NULL, NULL);
 		icon = g_themed_icon_new (NAUTILUS_ICON_DESKTOP);
 		last_iter = add_place (sidebar, PLACES_BUILT_IN,
@@ -1780,23 +1777,18 @@ drive_start_from_bookmark_cb (GObject      *source_object,
 static void
 open_selected_bookmark (NautilusPlacesSidebar *sidebar,
 			GtkTreeModel	      *model,
-			GtkTreePath	      *path,
+			GtkTreeIter	      *iter,
 			NautilusWindowOpenFlags	      flags)
 {
 	NautilusWindowSlot *slot;
-	GtkTreeIter iter;
 	GFile *location;
 	char *uri;
 
-	if (!path) {
+	if (!iter) {
 		return;
 	}
 
-	if (!gtk_tree_model_get_iter (model, &iter, path)) {
-		return;
-	}
-
-	gtk_tree_model_get (model, &iter, PLACES_SIDEBAR_COLUMN_URI, &uri, -1);
+	gtk_tree_model_get (model, iter, PLACES_SIDEBAR_COLUMN_URI, &uri, -1);
 
 	if (uri != NULL) {
 		DEBUG ("Activating bookmark %s", uri);
@@ -1828,7 +1820,7 @@ open_selected_bookmark (NautilusPlacesSidebar *sidebar,
 		GVolume *volume;
 		NautilusWindowSlot *slot;
 
-		gtk_tree_model_get (model, &iter,
+		gtk_tree_model_get (model, iter,
 				    PLACES_SIDEBAR_COLUMN_DRIVE, &drive,
 				    PLACES_SIDEBAR_COLUMN_VOLUME, &volume,
 				    -1);
@@ -1869,11 +1861,14 @@ open_shortcut_from_menu (NautilusPlacesSidebar *sidebar,
 {
 	GtkTreeModel *model;
 	GtkTreePath *path;
+	GtkTreeIter iter;
 
 	model = gtk_tree_view_get_model (sidebar->tree_view);
 	gtk_tree_view_get_cursor (sidebar->tree_view, &path, NULL);
 
-	open_selected_bookmark (sidebar, model, path, flags);
+	gtk_tree_model_get_iter (model, &iter, path);
+
+	open_selected_bookmark (sidebar, model, &iter, flags);
 
 	gtk_tree_path_free (path);
 }
@@ -2476,7 +2471,8 @@ bookmarks_key_press_event_cb (GtkWidget             *widget,
        event->keyval == GDK_KEY_space)) {
 
       GtkTreeModel *model;
-      GtkTreePath *path;
+      GtkTreeSelection *selection;
+      GtkTreeIter iter;
       NautilusWindowOpenFlags flags = 0;
 
       if ((event->state & modifiers) == GDK_SHIFT_MASK) {
@@ -2486,11 +2482,11 @@ bookmarks_key_press_event_cb (GtkWidget             *widget,
       }
 
       model = gtk_tree_view_get_model (sidebar->tree_view);
-      gtk_tree_view_get_cursor (sidebar->tree_view, &path, NULL);
+      selection = gtk_tree_view_get_selection (sidebar->tree_view);
+      gtk_tree_selection_get_selected (selection, NULL, &iter);
 
-      open_selected_bookmark (sidebar, model, path, flags);
+      open_selected_bookmark (sidebar, model, &iter, flags);
 
-      gtk_tree_path_free (path);
       return TRUE;
   }
 
@@ -2686,9 +2682,9 @@ bookmarks_button_release_event_cb (GtkWidget *widget,
 				   NautilusPlacesSidebar *sidebar)
 {
 	GtkTreePath *path;
+	GtkTreeIter iter;
 	GtkTreeModel *model;
 	GtkTreeView *tree_view;
-	gboolean ret;
 
 	path = NULL;
 
@@ -2697,9 +2693,10 @@ bookmarks_button_release_event_cb (GtkWidget *widget,
 	}
 
 	if (clicked_eject_button (sidebar, &path)) {
-		ret = eject_or_unmount_bookmark (sidebar, path);
+		eject_or_unmount_bookmark (sidebar, path);
 		gtk_tree_path_free (path);
-		return ret;
+
+		return FALSE;
 	}
 
 	tree_view = GTK_TREE_VIEW (widget);
@@ -2714,7 +2711,9 @@ bookmarks_button_release_event_cb (GtkWidget *widget,
 		gtk_tree_view_get_path_at_pos (tree_view, (int) event->x, (int) event->y,
 					       &path, NULL, NULL, NULL);
 
-		open_selected_bookmark (sidebar, model, path, 0);
+		gtk_tree_model_get_iter (model, &iter, path);
+
+		open_selected_bookmark (sidebar, model, &iter, 0);
 
 		gtk_tree_path_free (path);
 	}
@@ -2837,6 +2836,7 @@ bookmarks_button_press_event_cb (GtkWidget             *widget,
 	} else if (event->button == 2) {
 		GtkTreeModel *model;
 		GtkTreePath *path;
+		GtkTreeIter iter;
 		GtkTreeView *tree_view;
 		NautilusWindowOpenFlags flags = 0;
 
@@ -2847,6 +2847,7 @@ bookmarks_button_press_event_cb (GtkWidget             *widget,
 
 		gtk_tree_view_get_path_at_pos (tree_view, (int) event->x, (int) event->y, 
 					       &path, NULL, NULL, NULL);
+		gtk_tree_model_get_iter (model, &iter, path);
 
 		if (g_settings_get_boolean (nautilus_preferences,
 					    NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
@@ -2857,7 +2858,7 @@ bookmarks_button_press_event_cb (GtkWidget             *widget,
 			flags = NAUTILUS_WINDOW_OPEN_FLAG_CLOSE_BEHIND;
 		}
 
-		open_selected_bookmark (sidebar, model, path, flags);
+		open_selected_bookmark (sidebar, model, &iter, flags);
 
 		if (path != NULL) {
 			gtk_tree_path_free (path);
@@ -3212,7 +3213,7 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 				  G_CALLBACK(desktop_setting_changed_callback),
 				  sidebar);
 
-	g_signal_connect_swapped (gnome_background_preferences, "changed::" NAUTILUS_PREFERENCES_DESKTOP_IS_HOME_DIR,
+	g_signal_connect_swapped (gnome_background_preferences, "changed::" NAUTILUS_PREFERENCES_SHOW_DESKTOP,
 				  G_CALLBACK(desktop_setting_changed_callback),
 				  sidebar);
 
